#!/usr/local/bin/perl
#
#fwlogstat.pl : Program to Analyze Checkpoint FW1(V4.0) Account Logs.
#
#    Copyright (C) 1999,2000  Rajeev Kumar
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Rajeev Kumar (rxknh@yahoo.com)
#
#Obtain Latest version of this software from:
# http://www.geocities.com/SiliconValley/Bit/9363/
################################################################################
#
# This program is written in parts, so one invocation of this program will 
#run only paricular section from below.
#
#PART 1: Command line invocation
#        o Invoke this program from command line, to convert RAW checkpoint logs
#          into processed logs. During this part, it will sort out account logs 
#          based on origin Firewall, create cache file for CKP services, users in
#          log resolved name for IP address found in logs etc.
#
#    fwlogstat-1.0/fwlogstat.pl <account-text-log-file>
#
#PART 2: Present main HTML form to select Firewall to be analyze the logs.
#
#PART 3: Allows users to select the search criteria from HTML form presented, after
#        selecting a particular firewall in Part 2.
#
#PART 4: Will process logs and present summary based on selections in Part 3.
#       
################################################################################
#Assuming the  Checkpoint Accounting logs in form of:
#
#num;logdate;logtime;orig;type;action;alert;i/f_name;i/f_dir;proto;src;dst;service;s_port;rule;elapsed;start_time;packets;bytes;user;sys_msgs
#0;12Nov1999; 7:59:21;192.233.231.226;account;accept;;daemon;inbound;udp;192.104.61.181;192.233.228.34;ntp-udp;ntp-udp;6;0:00:00;12Nov1999  7:58:41;2;152;user1;
#1;12Nov1999; 7:59:22;192.233.231.226;account;accept;;daemon;inbound;udp;212.209.24.33;192.233.228.34;ntp-udp;ntp-udp;6;0:00:00;12Nov1999  7:58:42;2;152;;
#
#NOTE: The user part may be missing if you don't select account log users from CKP 
#      rule selections base.=> weired :( 
#Accounting Logs can be huge, (Example: 200,000+ lines in 24 hours) to process such a
#huge file is a time consuming process and opening a huge file may use swap which will
#Create further delay. In order to check this, Specify the limit below how many line a
#processed log file should contain after that it will start another log file.
#
#Processed Log file name(syntax):  log_<number>_starttime_endtime    
#                                   times are in UTC(GMT)
##################################################################################
$ver="fwlogstat-1.0";
$this_script="fwlogstat.pl";   #No need to modify this.

##################### USER MODIFIABLE SECTION :Modify few variables below according to your site###########
#
$cgi_bin_dir="/usr/local/apache/cgi-bin";
$url="http://www.yourwebserver.com";
$this_cgi="$url/cgi-bin/$ver/$this_script";    #May leave default value.
$html_title="Checkpoint Firewall Accounting Log (Your company  Inc..)";

#Mentions All origin firewalls, you are expecting in account logs. IP => 'name' pair.
#if you miss any firewall below, no problem all it will create log directory based on
#IP address. and also uses in HTML output. (Add each Firewall in each line like below)
#So suppose we are getting account logs from two firewall on Master :
# 192.168.0.1  named enterprise-fw
# 192.168.1.1 named as remote-fw so we define a hash below.
#Replace IP address and name as with your firewall(s), if you have more firewall to add
#add/replace the hash elements below.

%firewall=(
	   '192.168.0.1' =>     'enterprise-fw',
	   '192.168.1.1' =>     'remote-fw',
	  );


############################### May leave default value below this ######################################
$location_this_script="$cgi_bin_dir/$ver/$this_script";
#Specify the working directory, should be read/writable by this script and atleast
#readble by your web server deamon(cgi-bin user).
$working_dir="$cgi_bin_dir/$ver/fwlogs";    

$fwlog_limit_per_file=50000;   #Limits these many logs per file, after that create new file. Must be >0

#DNS resolution for IP (src/dst hosts) is ON by default, dns cache will be created 
#at raw log processing time, specify how long that cache to be kept.
$dns_check=1;
$dns_ttl=30*86400;   #TTL value(Days*secs_in_one_day) for DNS records. After that new file is created.
$tmp_dir="/usr/tmp"; #Should be readble/writable by cgi-bin user. 
################################ USER SERVICABLE PART ENDS HERE##########################################


push(@INC,"$cgi_bin_dir/$ver/lib");   #For additonal Lib/Modules
require "cgi-lib.pl";
use AnyDBM_File;         #Use Any best DBM database pacakges for this script

require Date::Manip;     #Use for Date manipulation, like getting UTC for random time
                         #Get Date::Manip package install it.  see man Date::Manip
                         #Can Use   use Date::Manip format also, but that require @INC
                         #variable modified at the Commandline(just puch(@INC,..) didn't work.
                         #probably Bug in this version of Perl
&Date::Manip::Date_Init();  #Run Date_Init() function just to intialize Date::Manip 
#--------------------------------------------------------------------------------------------------------

$dns_file="$working_dir/dns_cache";           #DBM file for DNS cache.
$fwlog_dir="$working_dir";
$nextrun="nextrun";
$active="active";
$ckp_sorted_log="ckp_rawlog";
$fwlog_lock_file="$working_dir/fwlog.lock";
$ckp_services_file="$working_dir/ckp_service_cache";  #All known Checkpoint Services(created automatically and updated at each log proccesing)
$std_services_txt="$working_dir/std_services_txt";    #Text version of Standard services file(http://www.isi.edu/in-notes/iana/assignments/port-numbers)
$std_services_dbm="$working_dir/std_services_dbm";    #DMB version of Standard services file(faster access).
$ckp_users_file="$working_dir/ckp_user_cache";        #All users so far reported in Log files (Updated at each log processing)

$ckp_raw_header="num;date;time;orig;type;action;alert;i/f_name;i/f_dir;proto;src;dst;service;s_port;rule;elapsed;start_time;packets;bytes;user;sys_msgs";

$temp_summ_file="$tmp_dir/ckp_summ_temp_$$";    #Temp. file to store selected log data to print later.

@protocol=('tcp','udp','icmp','rcp');    #All possible protocols

$unix_move="/bin/mv -f ";

###################################################################################################################


$|=1;  #Flush all I/O as soon as possible.
open (STDERR, ">&STDOUT");  #Just for debugging

&ReadParse(*fwlog);    #Check to see if this is called from HTML form

############################################################################################
############################### PART 1 : Command Line Invocation section ###################
############################################################################################

###########################################################################################
# PART 1A : Command line invocation:( But if not 1 or 2 argument supplied   ###############
###########################################################################################
if( ($fwlog{instance} =~ /^$/) && (($#ARGV lt 0) || ($#ARGV gt 1) ) ) #If not one or two argument notsupplied(command line invocation)
  {
    print "Usage:  $this_script  CKP_fw_account_file \n\t\t OR\n";  
    print "        $this_script  -f FW_IP\n";
    exit(1);
    #We can check this exit value status
  }

###########################################################################################
# PART 1B : Command line invocation:(only one argument, process RAW log file from CKP)#####
#           In this section sort out logs based on ORIGIN(Firewall)                   #####
###########################################################################################
if( ($fwlog{instance} =~ /^$/) && ($#ARGV eq 0) ) 
  {
    $fw_rawlog=$ARGV[0];
    if( !(-d "$fwlog_dir") )   #If this is very first invocation , create sorted log directory
      {
	mkdir("$fwlog_dir",0755) || &cleanup("Unable to create Log directory \"$fwlog_dir\"","$fwlog_lock_file");
      }
    print "###########################################################################\n";
    print "# Sorting Checkpoint Raw Accounting Logs as per ORIGIN(Firewall)          #\n";
    print "###########################################################################\n";
    open(IN,"<$fw_rawlog") || &cleanup("Unable to open file \"$fw_rawlog\"","$fwlog_lock_file");
    while($_=<IN>)
      {
	if(!(/^$/ || /^[a-zA-Z]/) && /^[0-9]/)   #SKIP Loop; If starts with number only (NO blankline || starts with alphabet)
	  {
	    ($num,$logdate,$logtime,$orig,$type,$action,$alert,$if_name,$if_dir,$proto,$src,$dst,$service,$s_port,$rule,$elapsed,$start_time,$packets,$bytes,$user,$sys_msgs)=split(";");   chomp($user); chomp($sys_msgs);

	    if( !(defined $fwrawlog{$orig}) )   #If this is new ORIGIN. Open sorted log file
	      { 
		$fwrawlog{$orig} = ($firewall{$orig} ne "") ? $firewall{$orig} : $orig; 
		$ckp_rawlog{$fwrawlog{$orig}}=0;   #Define this for check later
	      }
	    
	    if( !(-d "$fwlog_dir/$fwrawlog{$orig}") )
	      { mkdir("$fwlog_dir/$fwrawlog{$orig}",0755) || &cleanup("Unable to create $fwrawlog{$orig}=>Log directory \"$fwlog_dir/$fwrawlog{$orig}\"","$fwlog_lock_file"); }
	    if($ckp_rawlog{$fwrawlog{$orig}} == 0)  #If the sorted CKP raw log file for this origin not yet created.
	      {
		#Create raw log file here.
		$ckp_rawlog_ptr{$fwrawlog{$orig}}="OUT_$fwrawlog{$orig}";
		$dummy_ptr=$ckp_rawlog_ptr{$fwrawlog{$orig}}; $dummy_file="$fwlog_dir/$fwrawlog{$orig}/$ckp_sorted_log"; 
		open($dummy_ptr,">$dummy_file") || &cleanup("Unable to create Log File \"$dummy_file\"","$fwlog_lock_file");
		print "\nLog for Firewall \"$fwrawlog{$orig}\" detected\n ..";
		print $dummy_ptr "$ckp_raw_header\n";
		$ckp_rawlog{$fwrawlog{$orig}}=1;  #File opened for this origin
	      }
	    print ".";     #Just print a dot
	    select("$ckp_rawlog_ptr{$fwrawlog{$orig}}");  #Select appropriate OUTPUT file.
	    print  $_;                                     #Print to selected OUTPUT file
	    select(STDOUT);
	  }                                #Select Standard INPUT back.
      }  #While loop ends here
    close(IN);
    #Close all OUT files too.
    foreach $o (values %ckp_rawlog_ptr)
      { close($o) }
    #Now invoke this script again for each Firewall Raw logfile.
    foreach $f (keys %fwrawlog)   #Get all firewalls detected so far in this invocation.
      {
	print "\n\n\n\n################ Coverting Raw Log for Firewall \"$f\" ################\n";
	system("$location_this_script -f $f");
      }
    #Check if Standard Services(other than CKP defined) text file exists, create DBM out of that.
    if( -T "$std_services_txt")  #If text services file exists, create DBM out of that
      {
	open(SERV,"<$std_services_txt") || print "Unable to open TEXT services file, IGNORING..\n";
	dbmopen(%std_services,"$std_services_dbm",0666) || print "Unable to open Services DBM file, IGNORING..\n";
	while($_=<SERV>)
	  {
	    #Each record will be in form of(example)
	    # domain           53/tcp    Domain Name Server

	    if( !( (/^\#/) || (/^$/) )  )     #If line doesn't start with # sign(i.e not a comment) or not blank
	      {
		($service_name,$service_number,$junk)=split;  
		($service_number,$junk)=split("/",$service_number);
		$std_services{$service_number}="$service_name";
	      }
	  } #while loop ends.
	close(SERV);
	dbmclose(%std_services);
      }  #services TXT->DBM section ends here.

    exit(0);   #Normal Termination
  } #Raw log --> sorted log section ends here

####################################################################################################
# PART 1C : Command line invocation:(TWO argument, First: firewallIP(FW_IP)  Second: Logdir for FW
####################################################################################################
if(($fwlog{instance} =~ /^$/) && ($#ARGV eq 1 ) )  #If this is command line invocation(2 arguments)
  {
    if($ARGV[0] ne "-f") 
      {
	print "Usage:  $this_script  CKP_fw_account_file \n\t\t OR\n";  
	print "        $this_script  -f FW_IP\n";
	exit(1);
	#We can check this exit value status    print    $firewall_IP=$ARGV[0];
      }
   
    $fwIP=$ARGV[1];
    $fwname = ($firewall{$fwIP} ne "") ? $firewall{$fwIP} : $fwIP;
    $ckp_log_dir="$working_dir/$fwname";

    if( !(-d "$ckp_log_dir") )  #Log directory for Firewall doesn't exist
	{ &cleanup("Log Directory for Firewall $fwname => $ckp_log_dir doesn't exist","$fwlog_lock_file"); }
	  
    $ckp_log_file="$ckp_log_dir/$ckp_sorted_log";
    $fwlog_next_file="$ckp_log_dir/$nextrun";
    $ckp_log_active="$ckp_log_dir/$active";
	
    if(-e "$fwlog_lock_file")  #If some other log processing going on.
      {
	print "\n#################################################################\n";
	print "Warning:: Log processing Busy by other invocation. Please wait to\n";
	print "          finish previous processing.\n";
	print "Lock file \"$fwlog_lock_file\" exists.\n";
	print "DO NOT DELETE LOCKFILE UNLESS YOU ARE VERY SURE.\n";
	print "#################################################################\n";
	exit(1);
      }
    open(TEST,">$fwlog_lock_file") || die "Unable to create Lock file $fwlog_lock_file $!";    #Create Lock file.
    close(TEST);
    #Check some conditions.

    if(-T $fwlog_next_file)   #If this file(Text -T ) exists(contains number of next log file)
      {
	open(NUM,"<$fwlog_next_file") || &cleanup("Unable to open $fwlog_next_file","$fwlog_lock_file");
	$fwlog_number=<NUM>; chomp($fwlog_number);   #Read First line, number of next log file.
	$fwlog_nexttime=<NUM>; chomp($fwlog_nexttime); #Read what should be the next(or >) UTC time in log file
	if($fwlog_number =~ /[^0-9]/)    #If this contains any NON-DIGIT character.
	  {
	    print "Error: \"$fwlog_next_file\" is corrupt.\n";
	    &cleanup("$fwlog_lock_file");
	  }
	if($fwlog_nexttime =~ /[^0-9]/)    #If this contains any NON-DIGIT character.
	  {
	    print "Error: \"$fwlog_next_file\" is corrupt.\n";
	    &cleanup("$fwlog_lock_file");
	  }
	close(NUM);
      }
    else    #If this file doesn't exist.
      {
	print "Creating \"$fwlog_next_file\" for FIRST TIME log processing\n";
	print "ANY OLD LOG FILES IN AREA \"$fwlog_dir\" may be overwritten now or in future\n";
	open(NUM,">$fwlog_next_file") || &cleanup("Unable to open $fwlog_next_file for writing","$fwlog_lock_file");
	print NUM "1\n";  #Next Logfile number
	print NUM "0\n";    #Starttime in next log file should be greater than this.
	print NUM "0\n";    #Starttime in the present(last) written log file.
	$fwlog_number=1;
	$fwnexttime=0;
	close(NUM);
      }
	
    #Now read the RAW log file obtained from Checkpoint "fw logexport" command
    # and create processed Lod file(s), as per limit specified above ($fwlog_limit_per_file)
    #The OUTPUT processed log format would be in a print statement below:
print "<<< $ckp_log_file <<<<";
    open(IN,"<$ckp_log_file") || &cleanup("Unable to open file \"$ckp_log_file\"","$fwlog_lock_file");   #Read Raw data obtained from CKP
    
    #Initializing Variables; Latest Perl doesn't require this.
    $total_bytes=0;           $total_packets=0;        $total_elapsed=0;
    foreach $p (@protocol)
      {
	$total_bytes{$p}=0;   $total_packets{$p}=0;    $total_elapsed{$p}=0;
      }
    $total_names=0;  #Total Number of Hosts, in logs
   
    #Read Cache File.
    if(-T $ckp_services_file)   #If service cache exists, Read file and register services.
      {
     	open(SERV,"<$ckp_services_file") || print "Unable to open Service cache \"$ckp_services_file\". IGNORING..\n";
	while($_=<SERV>)
	  { 
	    chomp;   #Remove \n if any
	    $service_cache{$_}="1"; 
	  }  #Register Service for further use.
    	close(SERV);
      }
    
    if(-T $ckp_users_file)   #If users cache exists, Read file and register user.
      {
     	open(USR,"<$ckp_users_file") || print "Unable to open User cache \"$ckp_users_file\". IGNORING..\n";
	while($_=<USR>)
	  { 
	    chomp;   #Remove \n if any
	    $user_cache{$_}="1"; 
	  }  #Register user for further use.
    	close(USR);
      }

    if( (-B "$dns_file") || (-B "$dns_file.dir" && -B "$dns_file.pag") )   #If DBM dns database exists.
      {
	dbmopen(%name,"$dns_file",0666);   #Read-Write mode both.
	$dummy1=time(); $dummy2=$dummy1-$name{last_created};
	print "Current_time($dummy1) - DNS_cache_last_created($name{last_created})= $dummy2  [TTL= $name{ttl}]\n";
	if( ( (time()) - $name{last_created} ) > $name{ttl} )    #If DNS cache expired.
	  {
	    print "DNS cache expired, RECREATING NEW DNS CACHE FILE [TTL=$dns_ttl]\n";
	    dbmclose(%name);
	    unlink "$dns_file","$dns_file.dir","$dns_file.pag";
	    dbmopen(%name,"$dns_file",0666);  #Recreate DNS cache.
	    $name{ttl}=$dns_ttl;
	    $name{last_created}=time();
	  }
      }
    else  #Simply create (initiate DNS cache file.)
      {
	print "Creating NEW DNS CACHE FILE [TTL=$dns_ttl]\n";
	dbmopen(%name,"$dns_file",0666);
	$name{ttl}=$dns_ttl;
	$name{last_created}=time();        #Current UTC-GMT time.
      }

    $clog_count=0;   #Counting the number of current logs in processed file. 
    $clog_start_time=0;     $clog_end_time=0;
    while($_=<IN>)   #Start reading RAW log file.
      {
	if(!(/^$/ || /^[a-zA-Z]/) && /^[0-9]/)   #SKIP Loop; If starts with number only (NO blankline || starts with alphabet)
	  {	
	    if($clog_count >= $fwlog_limit_per_file)    #Time to open next log file
	      {
		close(LOG); #Close already opened active log file.
		#Move Log File ($clog_start_time and $clog_end_time are already caculated in previous loops
		$wlog="$ckp_log_dir/log_$fwlog_number\_$clog_start_time\_$clog_end_time";
		(system("$unix_move $ckp_log_active $wlog")) && &cleanup("Unable to move $ckp_log_active -> $wlog.","$fwlog_lock_file","$ckp_log_active");
		++$fwlog_number;                   #Increment the log file number
		$fwlog_nexttime=$time_utc;         #get from last value
		$clog_count=0;                    #Re-intialize this counter
		$clog_start_time=0;     $clog_end_time=0
	      }
	    if($clog_count == 0)                  #Open New processed  Log file
	      {
		open(LOG,">$ckp_log_active") || &cleanup("Unable to  write file \"$ckp_log_active\"","$fwlog_lock_file","$ckp_log_active"); #Open temp activelog
		print LOG "num;src;src_name;dst;dst_name_name;proto;s_port,service;bytes;packets;elapsed;time_utc;logdate;logtime;user\n";
	      }
	    
	    ++$clog_count;
	    ($num,$logdate,$logtime,$orig,$type,$action,$alert,$if_name,$if_dir,$proto,$src,$dst,$service,$s_port,$rule,$elapsed,$start_time,$packets,$bytes,$user,$sys_msgs)=split(";");   chomp($user); chomp($sys_msgs);
	    
	    #Check if this is a new service, not in service cache file. and starts with alphabet
	    if((!($service_cache{$service})) && ($service =~ /^[a-zA-Z]/))   #If service not in cache 
	      { $service_cache{$service}="1"; }
	   
	    #Based on source port
	    if((!($service_cache{$s_port})) && ($s_port =~ /^[a-zA-Z]/))   #If service not in cache
	      { $service_cache{$s_port}="1"; }
	    
	    #Check if this is a new user, not in user cache file.
	    if((!($user_cache{$user})) && ($user =~ /^[a-zA-Z]/))   #If this user is not in cache already
	      { $user_cache{$user}="1"; }
	    
	    #Converting time into seconds
	    ($hh,$mm,$ss)=split(":",$elapsed);
	    $elapsed=$hh*3600+$mm*60+$ss;
	    #Resolve IP->Hostname (sourceand destination IP) name;
	    if(!(defined $name{$src}))
	      {
		++$total_names;
		if($dns_check)
		  {
		    @ip=split(/\./,$src);
		    print "[$total_names]Resolving<$num> (SRC) $src ==> ";
		    ($name,$alias,$junk)=gethostbyaddr((pack('C4',@ip)),2); 
		    if ($name !~ /\w+/) {  $name="NOT_RESOLVED"; }
		    print " $name\n";
		  }
		else
		  {
		    $name="UNKNOWN";
		    print ".";
		  }
		$name{$src}=$name;
	      }
	    
	    if(!(defined $name{$dst}))
	      {
		++$total_names;
		if($dns_check)
		  {
		    @ip=split(/\./,$dst);
		    print "[$total_names]Resolving<$num> (DST) $dst ==> ";
		    ($name,$alias,$junk)=gethostbyaddr((pack('C4',@ip)),2);
		    if ($name !~ /\w+/) { $name="NOT_RESOLVED"; }
		    print " $name\n";
		  }
		else
		  {
		    $name="UNKNOWN";
		    print ".";
		  }
		$name{$dst}="$name";  
	      }
	    $ddmmmyyyy=$logdate;
	    $ddmmmyyyy=~/(\d+)([a-zA-Z]+)(\d+)/;            #Parse Date.
	    $dd=$1; $mmm=$2; $yyyy=$3;                        #Get Date Component
	    $mmm=&getintmonth($mmm);                            #Get Numeral Value for month
	    $hhmmss=$logtime;
	    ($hh,$mm,$ss)=split(/:/,$hhmmss);               #Get Time component
	    #Convert Locat time in GMT and store its UTC value, this will take care of Day light saving.
	    #as GMT value always remain consistent.
	    $time_utc=&Date::Manip::Date_SecsSince1970GMT($mmm,$dd,$yyyy,$hh,$mm,$ss);  #Used from Date::Manip module
	    #Use Full name including Package name, since we are usinf require (not use) to include
	    #perl module. Use will copy the name space, but require doesn't.
	    
	    if($time_utc < $fwlog_nexttime)  #If the time is less than the expected, means logs already processed or avoid this.
	      {
		close(LOG);
		#Local time first converts GMT(argument) into localtime.
		$t1=localtime($time_utc);   $t2=localtime($fwlog_nexttime);
		print "\n###################################################################################\n";
		print "ERROR: \n";
		print "Either this logfile \"$ckp_log_file\" has been processed already\n OR \n";
		print "Start Date(UTC) in this log file \"$time_utc($t1) < $fwlog_nexttime($t2) \". \n";
		print "Which means you are trying to process PAST log file. *NOT ALLOWED*\n";
		print "###################################################################################\n";
		#Cleanup before exit.
		&cleanup("Check Log files","$ckp_log_active","$fwlog_lock_file");
	      }

	    if($clog_count == 1)    #Reading First Log for fresh file
	      {
		$clog_start_time=$time_utc;
	      }
	    $clog_end_time=$time_utc;      #Just for use further, when fwlog_limit_per_file crossed	 or while loop exits
	    print LOG "$num;$src;$name{$src};$dst;$name{$dst};$proto;$s_port;$service;$bytes;$packets;$elapsed;$time_utc;$logdate;$logtime;$user\n";
	    #Get some basic summary;
	    $total_bytes += $bytes;    $total_packets += $packets;      $total_elapsed += $elapsed;
	    $total_bytes{$proto} += $bytes;  $total_packets{$proto} += $packets;  $total_elapsed{$proto} += $elapsed;
	    if(!(defined $service{$service}))   #If the service is not already defined
	      {
		$service{$service}='';     #Just define
		$service_bytes{$service}=0;  $service_packets{$service}=0;   $service_elapsed{$service}=0;
	      }
	    $service_bytes{$service} += $bytes; $service_packets{$service} += $packets;  $service_elapsed{$service} += $elapsed;
	  }
      } #While Loop Ends here (Reading RAW Log File
    #
    #Processing Complete.
    #Whatever Remaining log is there in active logfile, just rename it.
    close(IN);
    close(LOG);#Close already opened active log file.
    #Move Log File ($clog_start_time and $clog_end_time are already caculated in previous loops
    $wlog="$ckp_log_dir/log_$fwlog_number\_$clog_start_time\_$clog_end_time";
    (system("$unix_move $ckp_log_active $wlog")) && &cleanup("->Unable to move $ckp_log_active -> $wlog.\n","$fwlog_lock_file","$ckp_log_active");
    ++$fwlog_number;                   #Increment the log file number
    $fwlog_nexttime=$clog_end_time;  
    #Write this number in the file so that next processing will start from there onwards.
    open(NUM,">$fwlog_next_file") || &cleanup("Unable to open $fwlog_next_file for writing","$fwlog_lock_file");
    print NUM "$fwlog_number\n";
    print NUM "$fwlog_nexttime\n";
    print NUM "$clog_start_time\n";     #Start time of last log file. Used further in processing section.
    close(NUM);
    #Remove Lock File, i.e Declare END Processing
    unlink("$fwlog_lock_file");
    #Write updated cache files
    if((open(SERV,">$ckp_services_file")))     #Successful open() call return ZERO.
      {
	foreach $s (sort(keys %service_cache))
	  {
	    if($s =~ /\w/)   #Not blank
	      { print SERV "$s\n"; }
	  }
	close(SERV);
      }
    
    if((open(USR,">$ckp_users_file")))         #Successful open() call return ZERO.
      {
	foreach $u (sort(keys %user_cache))
	  {
	    if($u =~ /\w/) #not blank
	      { print USR "$u\n"; }
	  }
	close(USR);
      }

    # Creating Summary.

    print "$total_bytes  $total_packets  $total_elapsed\n";
    foreach $p (@protocol)
      {
	print "$p ==> $total_bytes{$p}  $total_packets{$p}  $total_elapsed{$p} \n";
      }
    foreach $s (keys %service)
      {
	print "$s ==> $service_bytes{$s} $service_packets{$s}  $service_elapsed{$s}\n";
      }
    
    print $total_names;
    dbmclose(%name);      #Close this hash now. 
  }
else             #If this is cgi-bin invocation
  {
    $header=&PrintHeader();
    $bottom=&HtmlBot();
    print $header;
    print "<TITLE>$html_title</TITLE>\n";

    
    ############################################################################################
    ############################### PART 2 : Printing Main Form              ###################
    ############################################################################################ 
    
    if($fwlog{action} =~ /main/i)
      {
	print <<MAIN01;
	 <H1> Checkpoint FW-1 (V4.0) Log Analysis </H1>
         <H2> Select Any  Firewall from below to analyze. </H2>
	   <FORM action=$this_cgi  method="post">
            <P>
             <INPUT type="hidden" name="instance" value="real">
MAIN01
       foreach $f (sort values %firewall)
	 {
	   print "<INPUT type=\"radio\" name=\"firewall\" value=\"$f\"> $f<BR>";
	 }
	
	print <<MAIN02;
	  </P><HR>
	   <INPUT type="submit"  name="action" value="search form">  
          </FORM><HR>
MAIN02
      }
	

    ############################################################################################
    ############################### PART 3 : Printing Search  Form           ###################
    ############################################################################################ 

    if($fwlog{action} =~ /search/i)
      {
	$ckp_log_dir="$working_dir/$fwlog{firewall}";    #Set Log directory for selected firewall
	$fwlog_next_file="$ckp_log_dir/$nextrun";

	#Get start and end time(for displaying in form) from last log file created.
	open(TIME,"<$fwlog_next_file") || print_error("Unable to open $fwlog_next_file");
	$junk=<TIME>; #Read first line.
	$end_t=<TIME>;   $start_t=<TIME>;   #Read 2nd and 3rd line as end and start time respectively
	#print "$start_t <===> $end_t";
        close(TIME);
	#localtime() assumes argument in GMT, so first convert into GMT -> localtime.
	($sec1,$min1,$hour1,$mday1,$mon1,$year1,$junk)=localtime($start_t);
	($sec2,$min2,$hour2,$mday2,$mon2,$year2,$junk)=localtime($end_t);
	$year1 += 1900; $mon1 += 1;  #month ranges 0..11, year value was after 1900  
	$year2 += 1900;  $mon2 += 1;    #localtime gives you $year , year after 1900(Confusion?? :-()
	print <<PRINTFORM01;
	 <H1> CheckPoint FW-1 (V4.0) Log Search Form ($fwlog{firewall}) </H1> <HR>
	 <FORM action=$this_cgi method="post">
          <P>
           <INPUT type="hidden" name="instance" value="real">
           <INPUT type="hidden" name="firewall" value="$fwlog{firewall}">
           <LABEL for="start_t"><b>Start_Time :&nbsp &nbsp</b> </LABEL>
           YYYY:<INPUT type="text" name="start_t_year" size="4" maxlength="4" value="$year1" >&nbsp&nbsp&nbsp;
           MM:<INPUT type="text" name="start_t_mon"     size="2" maxlength="2" value="$mon1">&nbsp&nbsp&nbsp;
           DD:<INPUT type="text" name="start_t_day"     size="2" maxlength="2" value="$mday1">&nbsp&nbsp&nbsp;
           HH:<INPUT type="text" name="start_t_hour"     size="2" maxlength="2" value="$hour1">
              <INPUT type="hidden" name="start_t_min" value="0">
              <INPUT type="hidden" name="start_t_sec" value="0">
           <BR>
           <LABEL for="end_t"><b>End__Time :&nbsp &nbsp</b> </LABEL>
           YYYY:<INPUT type="text" name="end_t_year" size="4" maxlength="4" value="$year2" >&nbsp&nbsp&nbsp
           MM:<INPUT type="text" name="end_t_mon"     size="2" maxlength="2" value="$mon2">&nbsp&nbsp&nbsp
           DD:<INPUT type="text" name="end_t_day"     size="2" maxlength="2" value="$mday2">&nbsp&nbsp&nbsp
           HH:<INPUT type="text" name="end_t_hour"     size="2" maxlength="2" value="$hour2">
              <INPUT type="hidden" name="end_t_min" value="0">
              <INPUT type="hidden" name="end_t_sec" value="0">
           <BR><HR>
	   <LABEL for="src_ip"><b>Source_IP(A.B.C.D):</b> </LABEL>
           <INPUT type="text" name="src_ip" size="15" maxlength="15">
           <LABEL for="dst_ip"><b>&nbsp&nbsp Destination_IP(A.B.C.D):</b> </LABEL>
	   <INPUT type="text" name="dst_ip"  size="15" maxlength="15"><BR>
           <b>Show Only Top Hosts Results:</b> <INPUT type=\"text\" name=\"num_top_IP\" size=4 value=\"10\"><BR>\n
	   <LABEL for="protocol"><b>Protocols : &nbsp &nbsp</b> </LABEL>
	   All<INPUT type="checkbox" name="all_proto" value="1" checked>&nbsp
PRINTFORM01
   #Printing Protocols Feild from the list of protocols
  foreach $p (@protocol)
    {
      print " $p<INPUT type=\"checkbox\" name=\"$p\" value=\"1\"> \&nbsp ";
    }
	open(IN,"<$ckp_services_file") || &print_error("Unable to open Service file $ckp_services_file");
	print <<PRINTFORM02;
         <BR><HR>
         <TABLE>
          <TR>
           <TD>
             <b>Source Port:</b><BR>(Select from CKP Defined Ports first)<BR><BR>
	     Unknown Source Port(Blank Separated):<INPUT type="text" name="unknown_src_ports"  size="25"><BR>
             &nbsp CKP Defined Source Port or ALL Source Ports:<BR> &nbsp &nbsp &nbsp &nbsp <SELECT multiple size="7" name="known_src_ports"> 
	     <OPTION selected value="all_ports">ALL PORTS</OPTION>
             <OPTION value="dummy">DUMMY</OPTION>
PRINTFORM02
	while($_=<IN>)
	  {
	    chomp;
	    print "<OPTION>$_</OPTION>\n";
	  }
	close(IN);
	print "</SELECT></TD>";

	open(IN,"<$ckp_services_file") || &print_error("Unable to open Service file $service_file");
	print <<PRINTFORM03;
         <TD>
          <b>Service(Destination) Port:</b><BR>(Select from Defined CKP services first)<BR><BR>
	   Unknown Service Port(Blank Separated):<INPUT type="text" name="unknown_dst_ports"  size="25"><BR>
           &nbsp CKP Defined Services or ALL Services:<BR> &nbsp &nbsp &nbsp &nbsp <SELECT multiple size="7" name="known_dst_ports"> 
	   <OPTION value="all_ports">ALL SERVICES</OPTION>
           <OPTION selected value="ckp_services">ALL CKP SERVICES</OPTION>
           <OPTION value="dummy">DUMMY</OPTION>
PRINTFORM03
	while($_=<IN>)
	  {
	    chomp;
	    $ckp_services_list .= " $_";
	    print "<OPTION>$_</OPTION>\n";
	  }
	close(IN);
	print "</SELECT></TD></TR></TABLE>";
	print "<b>Show Only Top Results:</b> <INPUT type=\"text\" name=\"num_top_results\" size=4 value=\"10\"><BR>\n";

	#print users Information
	open(IN,"<$ckp_users_file") || &print_error("Unable to open Users file $ckp_users_file");
	print <<PRINTFORM04;
           <HR>
	   <b>Users(From Internet):</b><BR>
            &nbsp &nbsp &nbsp <SELECT multiple size="4" name="internet_users"> 
	   <OPTION selected value="all_traffic">ALL TRAFFIC</OPTION>
           <OPTION          value="ckp_users">ALL USERS</OPTION>
PRINTFORM04
  while($_=<IN>)
    {
      chomp;
      $ckp_users_list .= " $_";
      print "<OPTION>$_</OPTION>\n";
    }
	close(IN);
	print "</SELECT>";
	
	print <<PRINTFORM05;
	   <HR>
	     <b>Print Long Summary:</b><INPUT type="checkbox" name="long_summ" value="1">
	   <HR>
           <INPUT type="hidden" name="ckp_services_list" value="$ckp_services_list">
           <INPUT type="hidden" name="ckp_users_list" value="$ckp_users_list">
           <INPUT type="submit" name="action"  value="Process Logs"> <INPUT type="reset" value="Reset Form"> <HR SIZE=10>
           </FORM>
PRINTFORM05
        print "<EM>Note: Logs will be processed as:(Default All for any variable) <BR></EM>";
	print " (Start_time --> End__Time) && Source_IP && Destination_IP && Protocols(tcp || udp || ....) && (Source Port &&  Destination Port) && Users";
      } #End Action=main m Here


    ############################################################################################
    ############################### PART 4 : Processing The Form inputs      ###################
    ############################################################################################

    if($fwlog{action} =~ /Process/i)     #Got Process Logs request
      {

	#$output=&PrintVariables(*fwlog);
	#print $output;
	$ckp_log_dir="$working_dir/$fwlog{firewall}";
	
	#ERROR-CHECK: If Date supplied is valid date
	if(!(&check_date($fwlog{start_t_year},$fwlog{start_t_mon},$fwlog{start_t_day},$fwlog{start_t_hour},$fwlog{start_t_min},$fwlog{start_t_sec})) ) 
	  { print_error("Wrong Date , Please Check Start Date"); }

	if(!(&check_date($fwlog{end_t_year},$fwlog{end_t_mon},$fwlog{end_t_day},$fwlog{end_t_hour},$fwlog{end_t_min},$fwlog{end_t_sec})) )
	  { print_error("Wrong Date , Please Check End Date"); }

	#Based on Time range supplied in form, we will select the log files. (UTC-GMT)
	$form_start_t=&Date::Manip::Date_SecsSince1970GMT($fwlog{start_t_mon},$fwlog{start_t_day},$fwlog{start_t_year},$fwlog{start_t_hour},$fwlog{start_t_min},$fwlog{start_t_sec});
	$form_end_t=&Date::Manip::Date_SecsSince1970GMT($fwlog{end_t_mon},$fwlog{end_t_day},$fwlog{end_t_year},$fwlog{end_t_hour},$fwlog{end_t_min},$fwlog{end_t_sec});
	if($form_start_t >= $form_end_t)
	  { print_error("Check you time range for log processing.Start time *MUST* be greater than end time"); }

	$min_start_t=$form_end_t;      #To start with(initial value assigned)
	                               #These variable is defined, such that if user specify out of range start time 
                                       #(very low for whic log file in not present) then this will store the min start 
                                       #time from which log files are
	                               #are processed. min_end time is not necessary since the form always present the
	                               #the maximum avialable from last log file. This variable is just for information purpose,
                                       #it won't affect any calculation downstream.
	
	#Select files based on time range.
	$num_read_fwlog=0;
	opendir(DIR,$ckp_log_dir) || &print_error("Unable to open Logs directory $ckp_log_dir");
	while($logfile=readdir(DIR))   #Read Directory content.
	  {
	    chomp($logfile);
	    if($logfile =~ /^log_\d+_\d+_\d+/)  #Example::  log_4_942734709_942754224
	      {
		($junk,$log_num,$log_start_t,$log_end_t)=split('_',$logfile);
		#print "$junk,$log_num,$log_start_t,$log_end_t <==> $form_start_t $form_end_t<BR>";
		#We will check if $log_start_t or $log_end_t lies in between $form_start_t <--> $form_end_t
		#                                       or
		# if $form_start_t or $form_end_t lies in between $log_start_t <--> $log_end_t 
		#if this condition satisfying means that log file should be included.
		#Example:  $form_start_t  <= log_start_t  <= $form_end_t   or $form_start_t <= $log_end_t <= $form_end_t
                #                                     or
                #          $log_start_t <= $form_start_t <= $log_end_t     or $log_start_t <= $form_end_t <= $log_end_t
		if( ( (($log_start_t >= $form_start_t) && ($log_start_t <= $form_end_t)) || (($log_end_t >= $form_start_t) && ($log_end_t <= $form_end_t)) ) || ( (($form_start_t >= $log_start_t) && ($form_start_t <= $log_end_t)) || (($form_end_t >= $log_start_t) && ($form_end_t <= $log_end_t)) ) )
		  { 
		    #print "<BR><==> THIS FILE $logfile<BR> ($form_start_t -- $form_end_t) --- $log_start_t>>><BR>";
		    $read_fwlog{$logfile}="1";   #Register this file for later processing.
		    ++$num_read_fwlog;
		    if($min_start_t > $log_start_t )
		      { $min_start_t = $log_start_t; }   #This will find log_start_t(minimum) of all log file processed.
		  }
	      }
	  }
	closedir(DIR);

	#If even now time falls out of range(less than lowest time log file avial.
        #(and also if it changed from initial value assigned)
	if(($form_start_t < $min_start_t) && ($min_start_t != $form_end_t))  
	  { 
	    $summ{actual_start_t}=localtime($min_start_t);   #Getting variable for HTML  Summary
	  }
	

	if($num_read_fwlog == 0)  #If no log files have been found to process.
	  {
	    print_error("No Logfile avialable for processing in the TIME range specified");
	  }

	#From <SELECT> option in FORM if you slect multiple choices the output will come \0 separated
	#for each key. like :(Example)
	# fwlog{known_src_port}=ntp-udp\0http\0smtp
	#so for such variables make sure we resolve this before testing any condition further.
	#Get All Source ports from Form(Unknown(blank separated) and Known(\0 separated)
	if($fwlog{known_src_ports} !~ /all_ports/i)   #If ALL is not selected
	  {
	    @unknown_src_ports=split(' ',$fwlog{unknown_src_ports});
	    @known_src_ports=split("\0",$fwlog{known_src_ports});
	    @src_ports=(@unknown_src_ports,@known_src_ports);
	    foreach $p (@src_ports)
	      {
		$form_src_ports{$p}="1";    #Just register the port/service
	      }
	    #print " @src_ports";
	  }
	#Similarily do for dst_ports(services)
	if($fwlog{known_dst_ports} !~ /all_ports/i)   #If ALL is not selected
	  {
	    if($fwlog{known_dst_ports} =~ /ckp_services/)  #If  all CKP services selected
	      {
		@dst_ports=split(' ',$fwlog{ckp_services_list});
	      }
	    else    #Only Selected Services.
	      {
		@unknown_dst_ports=split(' ',$fwlog{unknown_dst_ports});
		@known_dst_ports=split("\0",$fwlog{known_dst_ports});
		@dst_ports=(@unknown_dst_ports,@known_dst_ports);
	      }
	    
	    foreach $p (@dst_ports)
	      {
		$form_dst_ports{$p}="1";    #Just register the port/service
	      }
	    #print "== @dst_ports";
	  }
	#Get all Users if selected other than ALL
	if($fwlog{internet_users} !~ /all_traffic/i)   #If ALL is not selected
	  {
	    if($fwlog{internet_users} =~ /ckp_users/) #If all CKP users selected
	      {
		@internet_users=split(' ',$fwlog{ckp_users_list});
	      }
	    else   #Only selected users
	      {
		@internet_users=split("\0",$fwlog{internet_users});
	      }

	    foreach $u (@internet_users)
	      {
		$form_internet_users{$u}="1";    #Just register the port/service
	      }
	    #print "== @internet_users";
	  }

	#Initializing Variables; Latest Perl doesn't require this.
	$total_bytes=0;           $total_packets=0;        $total_elapsed=0;
	foreach $p (@protocol)
	  {
	    $total_bytes{$p}=0;   $total_packets{$p}=0;    $total_elapsed{$p}=0;
	  }
	#Check What protocols are checked in the Form
	foreach $p (@protocol)
	  {
	    if($fwlog{$p} eq "1") { $form_protocol{$p}="1"; }   #Simply make listing
	  }

	#See if Long summary is requested, simply open temp file for later use.
	if($fwlog{long_summ} eq "1")
	  { open(TMP_SUMM,">$temp_summ_file") || print_error("Unable to Open(for write) Temporary Summary file $temp_summ_file"); }

	#Now run conditions over each log files marked for processing with %read_fwlog (above) and process data.
	print "Total Log Files to be processed = $num_read_fwlog <BR>\n";
	foreach $inlog (sort keys %read_fwlog)     #Log File Processing Starts (foreach loop)
	  {
	    $inlog="$ckp_log_dir/$inlog";
	    print "Processing File = $inlog <BR>\n";
	    
	    open(IN,"<$inlog") || print_error("Unable to open Logs $inlog");
	    $_=<IN>;    #Gobble Header line.
	    while($_=<IN>)      #Processing Logs in current Log file
	      {
		$process_record=1;
		($num,$src,$name_src,$dst,$name_dst,$proto,$s_port,$service,$bytes,$packets,$elapsed,$time_utc,$logdate,$logtime,$user)=split(/;/,$_);
		chomp($user);   #Delete \n character
		#Check if the record needs to be included in this caculation.
	      SWITCH1:  #Stop Processing if record is not meeting conditions.
		{
		  if(($fwlog{src_ip} =~ /\w+/) && ($fwlog{src_ip} ne $src)) { $process_record=0; last SWITCH1; }
		  if(($fwlog{dst_ip} =~ /\w+/) && ($fwlog{dst_ip} ne $dst)) { $process_record=0; last SWITCH1; }
		  if(($fwlog{all_proto} ne "1") && ($form_protocol{$proto} ne "1")) { $process_record=0; last SWITCH1; }
		  if(($fwlog{known_src_ports} !~ /all_ports/i) && ($form_src_ports{$s_port} ne "1")) { $process_record=0; last SWITCH1; }
		  if(($fwlog{known_dst_ports} !~ /all_ports/i) && ($form_dst_ports{$service} ne "1")) { $process_record=0; last SWITCH1; }
		  #See if logtime is not in the range specified in query form. reject record
		  #NEGATE !($form_start_t <= time_utc <= $form_end_t) 
		  if(!( ($time_utc >= $form_start_t) && ($time_utc <= $form_end_t) ) ) {$process_record=0; last SWITCH1; }
		  if(($fwlog{internet_users} !~ /all_traffic/i) && ($form_internet_users{$user} ne "1")) { $process_record=0; last SWITCH1; }
		}
		if($process_record)
		  {
		    #print "<BR>$total_bytes($bytes)";
		    $total_bytes += $bytes;    $total_packets += $packets;   $total_elapsed += $elapsed;
		    $total_bytes{$proto} += $bytes;  $total_packets{$proto} += $packets;  $total_elapsed{$proto} += $elapsed;
		    if(!(defined $service{$service}))   #If the service is not already defined
		      {
			$service{$service}="1";     #Just define
			$service_bytes{$service}=0;  $service_packets{$service}=0;   $service_elapsed{$service}=0;
		      }
		    $service_bytes{$service} += $bytes; $service_packets{$service} += $packets;  $service_elapsed{$service} += $elapsed;
		    if(!(defined $srcIP{$src}))  #If this IP address in source is not registered so far
		      {
			$srcIP{$src}="$name_src";  #Register.
			$srcIP_bytes{$src}=0; $srcIP_packets{$src}=0; $srcIP_elapsed{$src}=0;
		      }
		    $srcIP_bytes{$src} += $bytes;  $srcIP_packets{$src} += $packets;  $srcIP_elapsed{$src} += $elapsed;

		    if(!(defined $dstIP{$dst}))  #If this IP address in source is not registered so far
		      {
			$dstIP{$dst}="$name_dst";  #Register.
			$dstIP_bytes{$dst}=0; $dstIP_packets{$dst}=0; $dstIP_elapsed{$dst}=0;
		      }
		    $dstIP_bytes{$dst} += $bytes;  $dstIP_packets{$dst} += $packets;  $dstIP_elapsed{$dst} += $elapsed;

		    #Add in if IP appear either in source and destination, so addting twice for same record here.
		    #$allIP_bytes{$src} += $bytes;  $allIP_packets{$src} += $packets;  $allIP_elapsed{$src} += $elapsed;
		    #$allIP_bytes{$dst} += $bytes;  $allIP_packets{$dst} += $packets;  $allIP_elapsed{$dst} += $elapsed;

		    #Store this record in a temp summary file if requested.
		    if($fwlog{long_summ} eq "1")
		      { print TMP_SUMM $_; }
		  }
		#print "---> $total_bytes"
		
	      } #while loop ends
	    close(IN);
	  }  #foreach loop end. (Log file processing loop end, means all log files have been processed.
	close(TMP_SUMM);   #Close temp. summary file, if opened earlier, else ignore

	#Now Sort out the data.
	#Look for sort function description (Programmin Perl, O'Reilly Camel Book, PP:217, Second Ed.)
	#sort functions takes value as two argument $a and $b, here we define block, and sorting in desceding
	#order i.e $b comes before $a in <=>(numerical comp. function)
	@unsorted=keys %service_bytes;
	@sort_service_bytes=sort { $service_bytes{$b} <=> $service_bytes{$a} } @unsorted;
	@unsorted=keys %service_packets;
	@sort_service_packets=sort { $service_packets{$b} <=> $service_packets{$a} } @unsorted;
	@unsorted=keys %service_elapsed;
	@sort_service_elapsed=sort { $service_elapsed{$b} <=> $service_elapsed{$a} } @unsorted;
	$top_num_services=( $fwlog{num_top_results}<($#unsorted+1) ? $fwlog{num_top_results} : ($#unsorted+1) ); #Get whichever is low. 

	@unsorted=keys %srcIP_bytes;
	@sort_srcIP_bytes=sort { $srcIP_bytes{$b} <=> $srcIP_bytes{$a} } @unsorted;
	@unsorted=keys %srcIP_packets;
	@sort_srcIP_packets=sort { $srcIP_packets{$b} <=> $srcIP_packets{$a} } @unsorted;
	@unsorted=keys %srcIP_elapsed;
	@sort_srcIP_elapsed=sort { $srcIP_elapsed{$b} <=> $srcIP_elapsed{$a} } @unsorted;
	$top_num_src=( $fwlog{num_top_IP}<($#unsorted+1) ? $fwlog{num_top_IP} : ($#unsorted+1) );

	@unsorted=keys %dstIP_bytes;
	@sort_dstIP_bytes=sort { $dstIP_bytes{$b} <=> $dstIP_bytes{$a} } @unsorted;
	@unsorted=keys %dstIP_packets;
	@sort_dstIP_packets=sort { $dstIP_packets{$b} <=> $dstIP_packets{$a} } @unsorted;
	@unsorted=keys %dstIP_elapsed;
	@sort_dstIP_elapsed=sort { $dstIP_elapsed{$b} <=> $dstIP_elapsed{$a} } @unsorted;
	$top_num_dst=( $fwlog{num_top_IP}<($#unsorted+1) ? $fwlog{num_top_IP} : ($#unsorted+1) );




	#Get search criteria just for information in Summary.
	$dummy=&gettextmonth($fwlog{start_t_mon});
	$summ{start_t}="($dummy/$fwlog{start_t_day}/$fwlog{start_t_year})<BR>$fwlog{start_t_hour}:$fwlog{start_t_min}:$fwlog{start_t_sec}Hrs";
	$dummy=&gettextmonth($fwlog{end_t_mon});
	$summ{end_t}="($dummy/$fwlog{end_t_day}/$fwlog{end_t_year})<BR> $fwlog{end_t_hour}:$fwlog{end_t_min}:$fwlog{end_t_sec}Hrs";
	
	if($fwlog{src_ip} =~ /\w+/) #If Source IP address/Net provided
	  { $summ{src_ip}=$fwlog{src_ip}; }
	else
	  { $summ{src_ip}="ANY"; }

	if($fwlog{dst_ip} =~ /\w+/) #If Destination  IP address/Net provided
	  { $summ{dst_ip}=$fwlog{dst_ip}; }
	else
	  { $summ{dst_ip}="ANY"; }
	
	if($fwlog{all_proto} ne "1") #If other Protocols specified
	  {
	    foreach $p (keys %form_protocol)
	      { $summ{proto} .= " $p,"; }
	  }
	else
	  { $summ{proto}="ANY"; }

	if($fwlog{known_src_ports} !~ /all_ports/i)  #If other source ports are defined
	  {
	    foreach $s (keys %form_src_ports)
	      { $summ{src_port} .= " $s,"; } 
	  }
	else
	  { $summ{src_port}="ANY"; }

	if($fwlog{known_dst_ports} !~ /all_ports/i)  #If other Service(Dst) ports are defined
	  {
	    foreach $d (keys %form_dst_ports)
	      { $summ{dst_port} .= " $d,"; } 
	  }
	else
	  { $summ{dst_port}="ANY"; }

	if($fwlog{internet_users} !~ /all_traffic/i)  #If users are  defined
	  {
	    foreach $u (keys %form_internet_users)
	      { $summ{internet_users} .= " $u,"; } 
	  }
	else
	  { $summ{internet_users}="ANY"; }



	#Converting data in more sesible form.
	$total_bytes /= (1024*1024);   #In MB
	$total_elapsed = &sec2hhmmss($total_elapsed);

	print <<SUMMARY01;
         <H1>SUMMARY: ($fwlog{firewall})</H1>\n<HR>\n
	 <TABLE  BORDER WIDTH="100%" NOSAVE>
	   <H2>Search Criteria:</H2>
	  <TR> 
           <TD BGCOLOR="#33CCFF"><b>Start Time:</b></TD>   <TD BGCOLOR="#FFCC99">$summ{start_t}<BR><HR> $summ{actual_start_t}</TD>
	   <TD BGCOLOR="#33CCFF"><b>End   Time:</b></TD>   <TD BGCOLOR="#FFCC99">$summ{end_t}</TD>
          </TR>
          <TR> 
	    <TD BGCOLOR="#33CCFF"><b>Source IP:</b></TD>    <TD BGCOLOR="#FFCC99">$summ{src_ip}</TD>
            <TD BGCOLOR="#33CCFF"><b>Destination IP:</b></TD>    <TD BGCOLOR="#FFCC99">$summ{dst_ip}</TD>
          </TR>
	  <TR>
	    <TD BGCOLOR="#33CCFF"><b>Protocol(s):</b></TD>       <TD BGCOLOR="#FFCC99">$summ{proto}</TD>
	    <TD BGCOLOR="#33CCFF"><b>Long Summary </b></TD>      <TD BGCOLOR="#FFCC99">No</TD>
	  </TR> 
          <TR>
            <TD BGCOLOR="#33CCFF"><b>Source Port(s):</b></TD>       <TD BGCOLOR="#FFCC99">$summ{src_port}</TD>
	    <TD BGCOLOR="#33CCFF" ><b>Dest. Port(Services):</b></TD>       <TD BGCOLOR="#FFCC99">$summ{dst_port}</TD>
	  </TR>
	  </TR> 
          <TR>
            <TD BGCOLOR="#33CCFF"><b>Internet Users:</b></TD>       <TD BGCOLOR="#FFCC99">$summ{internet_users}</TD>
	  </TR>
	 </TABLE>
	 
	   <HR>
	 
	  <TABLE BORDER WIDTH="100%" NOSAVE>
	   <H2>Total Data</H2>
	   <TR> 
            <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
            <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
            <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
           </TR>
	   <TR> 
            <TD BGCOLOR="#FFCC99">$total_bytes(MB)</TD> 
            <TD BGCOLOR="#FFCC99">$total_packets</TD> 
            <TD BGCOLOR="#FFCC99">$total_elapsed</TD>
           </TR> 
	  </TABLE>

           <HR>
	     
	   <TABLE BORDER WIDTH="100%" NOSAVE>
	    <H2>Protocol Level Summary.</H2>
	    <TR> 
             <TD BGCOLOR="#33CCFF"><b>Protocol</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
            </TR>
SUMMARY01
  #Printing Protocol Level Summary
  foreach $p (@protocol)
    {
      print "<TR>\n";
      print "<TD BGCOLOR=\"\#33CCFF\"><b>$p</b></TD>\n";
      $dummy=($total_bytes{$p}/(1024*1024));
      print "<TD BGCOLOR=\"\#FFCC99\">$dummy(MB)</TD>\n";
      print "<TD BGCOLOR=\"\#FFCC99\">$total_packets{$p}</TD>\n";
      $dummy=&sec2hhmmss($total_elapsed{$p});
      print "<TD BGCOLOR=\"\#FFCC99\">$dummy</TD>\n";
      print "</TR>\n";
    }
	print "</TABLE>\n<HR>\n";   #End Protocol Table here

################################ Service level(Dst Port) summary starts ########### 
	#See if Standard service(other than CKP dfined) file(DBM) exists, the read services from there also.
	if( (-B "$std_services_dbm") || ( (-B "$std_services_dbm.dir") && (-B "$std_services_dbm.pag") ) )
	  {
	    dbmopen(%std_services,"$std_services_dbm",0666) || print "Unable to open DBM services file..Ignoring ..<BR>\n";
	  }
	print <<SUMMARY02;
	  <TABLE BORDER WIDTH="100%" NOSAVE>
	    <H2>Service(Destination Port) Level Summary.</H2>
            <H3>Top $top_num_services , services based on BYTES transfer</H3>
	    <TR> 
             <TD BGCOLOR="#33CCFF"><b>Service<BR>(Destination Port)</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
            </TR>
SUMMARY02
	#Printing Service Level summary
	foreach($i=0 ; $i<$top_num_services  ; ++$i)
	  {
	    $s=$sort_service_bytes[$i];
	    if($std_services{$s} ne "")  #If this is standard service
	      { $dummy="$s($std_services{$s})"; }
	    else
	      { $dummy="$s"; }
		
	    print "<TR>\n";
	    print "<TD BGCOLOR=\"\#33CCFF\"><b>$dummy</b></TD>\n";
	    $dummy=($service_bytes{$s}/(1024*1024));
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy(MB)</TD>\n";
	    print "<TD BGCOLOR=\"\#FFCC99\">$service_packets{$s}</TD>\n";
	    $dummy=&sec2hhmmss($service_elapsed{$s});
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy</TD>\n";
	    print "</TR>\n";
	  }
	print "</TABLE>\n<HR>\n"; #End Service Level Table here.
	
	
	###############################################################################
	print <<SUMMARY03;
	<TABLE BORDER WIDTH="100%" NOSAVE>
	  <H3>Top $top_num_services , services based on PACKETS transfer</H3>
	   <TR> 
             <TD BGCOLOR="#33CCFF"><b>Service<BR>(Destination Port)</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
           </TR>
SUMMARY03
	#Printing Service Level(Packets) summary
	foreach($i=0 ; $i<$top_num_services  ; ++$i)
	  {
	    $s=$sort_service_packets[$i];
	    if($std_services{$s} ne "")  #If this is standard service
	      { $dummy="$s($std_services{$s})"; }
	    else
	      { $dummy="$s"; }
	    
	    print "<TR>\n";
	    print "<TD BGCOLOR=\"\#33CCFF\"><b>$dummy</b></TD>\n";
	    $dummy=($service_bytes{$s}/(1024*1024));
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy(MB)</TD>\n";
	    print "<TD BGCOLOR=\"\#FFCC99\">$service_packets{$s}</TD>\n";
	    $dummy=&sec2hhmmss($service_elapsed{$s});
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy</TD>\n";
	    print "</TR>\n";
	  }
	print "</TABLE>\n<HR>\n"; #End Service Level Table(Packets) here.	

	###############################################################################
	print <<SUMMARY04;
	<TABLE BORDER WIDTH="100%" NOSAVE>
	  <H3>Top $top_num_services , services based on TIME(ELAPSED)</H3>
	   <TR> 
             <TD BGCOLOR="#33CCFF"><b>Service<BR>(Destination Port)</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
           </TR>
SUMMARY04
	#Printing Service Level(elapsed time) summary
	foreach($i=0 ; $i<$top_num_services  ; ++$i)
	  {
	    $s=$sort_service_elapsed[$i];
	    if($std_services{$s} ne "")  #If this is standard service
	      { $dummy="$s($std_services{$s})"; }
	    else
	      { $dummy="$s"; }

	    print "<TR>\n";
	    print "<TD BGCOLOR=\"\#33CCFF\"><b>$dummy</b></TD>\n";
	    $dummy=($service_bytes{$s}/(1024*1024));
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy(MB)</TD>\n";
	    print "<TD BGCOLOR=\"\#FFCC99\">$service_packets{$s}</TD>\n";
	    $dummy=&sec2hhmmss($service_elapsed{$s});
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy</TD>\n";
	    print "</TR>\n";
	  }
	print "</TABLE>\n<HR>\n"; #End Service Level Table here.
	dbmclose(%std_services);
	################################ Service level(Dst Port) summary Ends ########### 

	############################ Host Level (Based on Source IP) STARTS  ####################
	print <<SUMMARY05;
	  <TABLE BORDER WIDTH="100%" NOSAVE>
	    <H2>Host (Based on Source) Level Summary.</H2>
            <H3>Top $top_num_src , HOSTS(Source:Originating Traffic) based on BYTES transfer</H3>
	    <TR> 
             <TD BGCOLOR="#33CCFF"><b>Originating Host</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
            </TR>
SUMMARY05
	#Printing Host(SRC) Level summary
	foreach($i=0 ; $i<$top_num_src  ; ++$i)
	  {
	    $s=$sort_srcIP_bytes[$i];
	    print "<TR>\n";
	    print "<TD BGCOLOR=\"\#33CCFF\"><b>$s($srcIP{$s})</b></TD>\n";
	    $dummy=($srcIP_bytes{$s}/(1024*1024));
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy(MB)</TD>\n";
	    print "<TD BGCOLOR=\"\#FFCC99\">$srcIP_packets{$s}</TD>\n";
	    $dummy=&sec2hhmmss($srcIP_elapsed{$s});
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy</TD>\n";
	    print "</TR>\n";
	  }
	print "</TABLE>\n<HR>\n"; #End
	
	
	###############################################################################
	print <<SUMMARY06;
	<TABLE BORDER WIDTH="100%" NOSAVE>
	  <H3>Top $top_num_src , HOSTS(Source:Originating Traffic) based on PACKETS transfer</H3>
	   <TR> 
             <TD BGCOLOR="#33CCFF"><b>Originating Host</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
           </TR>
SUMMARY06
	#Printing Host Level(SRC) starts
	foreach($i=0 ; $i<$top_num_src  ; ++$i)
	  {
	    $s=$sort_srcIP_packets[$i];
	    print "<TR>\n";
	    print "<TD BGCOLOR=\"\#33CCFF\"><b>$s($srcIP{$s})</b></TD>\n";
	    $dummy=($srcIP_bytes{$s}/(1024*1024));
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy(MB)</TD>\n";
	    print "<TD BGCOLOR=\"\#FFCC99\">$srcIP_packets{$s}</TD>\n";
	    $dummy=&sec2hhmmss($srcIP_elapsed{$s});
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy</TD>\n";
	    print "</TR>\n";
	  }
	print "</TABLE>\n<HR>\n"; #End 

	###############################################################################
	print <<SUMMARY07;
	<TABLE BORDER WIDTH="100%" NOSAVE>
	  <H3>Top $top_num_src , HOSTS(Source:Originating Traffic) based on TIME(ELAPSED)</H3>
	   <TR> 
             <TD BGCOLOR="#33CCFF"><b>Orginating Host</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
           </TR>
SUMMARY07
	#Printing Host level summary starts
	foreach($i=0 ; $i<$top_num_src  ; ++$i)
	  {
	    $s=$sort_srcIP_elapsed[$i];
	    print "<TR>\n";
	    print "<TD BGCOLOR=\"\#33CCFF\"><b>$s($srcIP{$s})</b></TD>\n";
	    $dummy=($srcIP_bytes{$s}/(1024*1024));
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy(MB)</TD>\n";
	    print "<TD BGCOLOR=\"\#FFCC99\">$srcIP_packets{$s}</TD>\n";
	    $dummy=&sec2hhmmss($srcIP_elapsed{$s});
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy</TD>\n";
	    print "</TR>\n";
	  }
	print "</TABLE>\n<HR>\n"; #End Service Level Table here.
	
	############################ Host Level (Based on Source IP) ENDS ####################
	
	#######################################################################################
	############################ Host Level (Based on Destination(Server) STARTS ##########
	#######################################################################################
	print <<SUMMARY08;
	  <TABLE BORDER WIDTH="100%" NOSAVE>
	    <H2>Host (Based on Destination{SERVER}) Level Summary.</H2>
            <H3>Top $top_num_dst , HOSTS(Destination:Server Traffic) based on BYTES transfer</H3>
	    <TR> 
             <TD BGCOLOR="#33CCFF"><b>Server Host</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
            </TR>
SUMMARY08
	#Printing Host(DST) Level summary
	foreach($i=0 ; $i<$top_num_dst  ; ++$i)
	  {
	    $s=$sort_dstIP_bytes[$i];
	    print "<TR>\n";
	    print "<TD BGCOLOR=\"\#33CCFF\"><b>$s($dstIP{$s})</b></TD>\n";
	    $dummy=($dstIP_bytes{$s}/(1024*1024));
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy(MB)</TD>\n";
	    print "<TD BGCOLOR=\"\#FFCC99\">$dstIP_packets{$s}</TD>\n";
	    $dummy=&sec2hhmmss($dstIP_elapsed{$s});
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy</TD>\n";
	    print "</TR>\n";
	  }
	print "</TABLE>\n<HR>\n"; #End
	
	
	###############################################################################
	print <<SUMMARY09;
	<TABLE BORDER WIDTH="100%" NOSAVE>
	  <H3>Top $top_num_dst , HOSTS(Destination:Server Traffic) based on PACKETS transfer</H3>
	   <TR> 
             <TD BGCOLOR="#33CCFF"><b>Server Host</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
           </TR>
SUMMARY09
	#Printing Host Level(DST) starts
	foreach($i=0 ; $i<$top_num_dst  ; ++$i)
	  {
	    $s=$sort_dstIP_packets[$i];
	    print "<TR>\n";
	    print "<TD BGCOLOR=\"\#33CCFF\"><b>$s($dstIP{$s})</b></TD>\n";
	    $dummy=($dstIP_bytes{$s}/(1024*1024));
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy(MB)</TD>\n";
	    print "<TD BGCOLOR=\"\#FFCC99\">$dstIP_packets{$s}</TD>\n";
	    $dummy=&sec2hhmmss($dstIP_elapsed{$s});
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy</TD>\n";
	    print "</TR>\n";
	  }
	print "</TABLE>\n<HR>\n"; #End 

	###############################################################################
	print <<SUMMARY10;
	<TABLE BORDER WIDTH="100%" NOSAVE>
	  <H3>Top $top_num_dst , HOSTS(Destination: Server Traffic) based on TIME(ELAPSED)</H3>
	   <TR> 
             <TD BGCOLOR="#33CCFF"><b>Server Host</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Total Bytes(MB)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Packets</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Total Connection Time(HH:MM:SS)</b></TD>
           </TR>
SUMMARY10
	#Printing Host level summary starts
	foreach($i=0 ; $i<$top_num_dst  ; ++$i)
	  {
	    $s=$sort_dstIP_elapsed[$i];
	    print "<TR>\n";
	    print "<TD BGCOLOR=\"\#33CCFF\"><b>$s($dstIP{$s})</b></TD>\n";
	    $dummy=($dstIP_bytes{$s}/(1024*1024));
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy(MB)</TD>\n";
	    print "<TD BGCOLOR=\"\#FFCC99\">$dstIP_packets{$s}</TD>\n";
	    $dummy=&sec2hhmmss($dstIP_elapsed{$s});
	    print "<TD BGCOLOR=\"\#FFCC99\">$dummy</TD>\n";
	    print "</TR>\n";
	  }
	print "</TABLE>\n<HR>\n"; #End Service Level Table here.

	#######################################################################################
	############################ Host Level (Based on Destination(Server) ENDS ##########
	#######################################################################################
	
	
	##################################################################################
	#Now print long Summary if requested, may be real time consuming thing.
	if($fwlog{long_summ} eq "1")
	  {
	    print <<LONGSUMM01;
	    <TABLE BORDER WIDTH="100%" NOSAVE>
	   <H2> Detailed Log Summary</H2> (May take very long time to print )
	   <TR> 
             <TD BGCOLOR="#33CCFF"><b>S.No.</b></TD>
             <TD BGCOLOR="#33CCFF"><b>SrcIP(name)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>DstIP(name)</b></TD> 
             <TD BGCOLOR="#33CCFF"><b>Proto</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Src_port</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Dst_port<BR>Service</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Bytes</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Packets</b></TD>
             <TD BGCOLOR="#33CCFF"><b>Elapsed Time</b></TD>
             <TD BGCOLOR="#33CCFF"><b>LogTime</b></TD>
             <TD BGCOLOR="#33CCFF"><b>User</b></TD>
           </TR>
LONGSUMM01
	    open(TMP_SUMM,"<$temp_summ_file") ||  print_error("Unable to Open(for read) Temporary Summary file $temp_summ_file");
	    $s_no=0;
	    while($_=<TMP_SUMM>)
	      {
		++$s_no;
		($num,$src,$name_src,$dst,$name_dst,$proto,$s_port,$service,$bytes,$packets,$elapsed,$time_utc,$logdate,$logtime,$user)=split(/;/,$_);
		print <<LONGSUMM02;
              	 <TR>
		  <TD BGCOLOR="#FFCC99"><FONT size="1">$s_no </FONT></TD>
                  <TD BGCOLOR="#FFCC99"><FONT size="1">$src(<FONT color="#FF0000">$name_src</FONT>)</FONT></TD> 
                  <TD BGCOLOR="#FFCC99"><FONT size="1">$dst(<FONT color="#0000FF">$name_dst</FONT>)</FONT></TD> 
                  <TD BGCOLOR="#FFCC99"><FONT size="1">$proto</FONT></TD>
                  <TD BGCOLOR="#FFCC99"><FONT size="1">$s_port</FONT></TD>
                  <TD BGCOLOR="#FFCC99"><FONT size="1">$service</FONT></TD>
                  <TD BGCOLOR="#FFCC99"><FONT size="1">$bytes</FONT></TD>
                  <TD BGCOLOR="#FFCC99"><FONT size="1">$packets</FONT></TD>
                  <TD BGCOLOR="#FFCC99"><FONT size="1">$elapsed</FONT></TD>
                  <TD BGCOLOR="#FFCC99"><FONT size="1">$logdate($logtime)</FONT></TD>
                  <TD BGCOLOR="#FFCC99"><FONT size="1">$user</FONT></TD>
                 </TR>
                 	</FONT> 
LONGSUMM02
	      }
	    print "</TABLE> <BR>";
	    close(TMP_SUMM);
	    unlink $temp_summ_file;   #Delete this now
	  } #Print long summary Ends here
###############################################################################################

      } #Action "Process Logs" end here

    print $bottom;
  }
  


#Subroutines

sub getintmonth
  {
    (local $text_mmm)=@_;
    (local $int_mmm);

  SWITCH:
    {
      if($text_mmm =~ /Jan/i)  {$int_mmm=1; last SWITCH;}
      if($text_mmm =~ /Feb/i)  {$int_mmm=2; last SWITCH;}
      if($text_mmm =~ /Mar/i)  {$int_mmm=3; last SWITCH;}
      if($text_mmm =~ /Apr/i)  {$int_mmm=4; last SWITCH;}
      if($text_mmm =~ /May/i)  {$int_mmm=5; last SWITCH;}
      if($text_mmm =~ /Jun/i)  {$int_mmm=6; last SWITCH;}
      if($text_mmm =~ /Jul/i)  {$int_mmm=7; last SWITCH;}
      if($text_mmm =~ /Aug/i)  {$int_mmm=8; last SWITCH;}
      if($text_mmm =~ /Sep/i)  {$int_mmm=9; last SWITCH;}
      if($text_mmm =~ /Oct/i)  {$int_mmm=10; last SWITCH;}
      if($text_mmm =~ /Nov/i)  {$int_mmm=11; last SWITCH;}
      if($text_mmm =~ /Dec/i)  {$int_mmm=12; last SWITCH;}
      $int_mmm=0;
    }
    return($int_mmm);
  }

sub gettextmonth
  {
    (local $int_mm)=@_;
    (local $text_mmm);

  SWITCH:
    {
      if($int_mm == 1)  {$text_mmm="Jan"; last SWITCH;}
      if($int_mm == 2)  {$text_mmm="Feb"; last SWITCH;}
      if($int_mm == 3)  {$text_mmm="Mar"; last SWITCH;}
      if($int_mm == 4)  {$text_mmm="Apr"; last SWITCH;}
      if($int_mm == 5)  {$text_mmm="May"; last SWITCH;}
      if($int_mm == 6)  {$text_mmm="Jun"; last SWITCH;}
      if($int_mm == 7)  {$text_mmm="Jul"; last SWITCH;}
      if($int_mm == 8)  {$text_mmm="Aug"; last SWITCH;}
      if($int_mm == 9)  {$text_mmm="Sep"; last SWITCH;}
      if($int_mm == 10)  {$text_mmm="Oct"; last SWITCH;}
      if($int_mm == 11)  {$text_mmm="Nov"; last SWITCH;}
      if($int_mm == 12)  {$text_mmm="Dec"; last SWITCH;}
      $text_mmm="NULL";;
    }
    return($text_mmm);
  }


sub print_error
  {
    local($message)=@_;
    local($bottom);

    print <<ERROR;
    <H1><FONT COLOR=\"\#ff0000\"><BLINK>ERROR :: </BLINK>$message</FONT> </H1>
ERROR
  print $bottom;
  exit(0);
  }

#convert seconds into hh.mm.ss
sub sec2hhmmss
  {
    local($seconds)=@_;
    local($hh,$mm,$ss,$hhmmss);
    
    $hh=0; $mm=0; $ss=0;
    if($seconds >= 3600) { $hh=int($seconds/3600);  $seconds %= 3600; }
    if($seconds >= 60)   { $mm=int($seconds/60);    $seconds %= 60;   }
    $ss=$seconds;
    $hhmmss="$hh:$mm:$ss";

    return($hhmmss);
  }


#Cleanup on exit.
#Syntax:  &cleanup("Cleanup Message","file1","file2",.....)
sub cleanup
  {
    (local $message)=shift @_;    #Get message (First argument)
    print "Error:: $message\n";
    foreach $f (@_)  #Each argument(file)
      {
	if( -e $f)  { unlink $f; }
      }
    exit(1);
  }


#This subroutine will verify the date entered is correct or not.
# &check_date($year,$mon,$day,$hour,$min,$sec);
#return 0 if wrong date(syntax) 1 if date is valid.
sub check_date
  {
    (local $year,$mon,$day,$hour,$min,$sec)=@_;
    (local $utc,$year1,$mon1,$day1,$hour1,$min1,$sec1,junk);
    #Convert this date into UTC(GMT) and convert back to local time
    # see if you get same values. If not the date syntax is wrong.
    #this is crude but good way to check.

    $utc=&Date::Manip::Date_SecsSince1970GMT($mon,$day,$year,$hour,$min,$sec);
    ($sec1,$min1,$hour1,$day1,$mon1,$year1,$junk)=localtime($utc);
    $year1 += 1900;  $mon1 += 1;   #Fix year and month
    
    if($year != $year1) {return 0;}
    if($mon != $mon1)   {return 0;}
    if($day != $day1)   {return 0;}
    if($hour != $hour1) {return 0;}
    if($min != $min1)   {return 0;}
    if($sec != $sec1)   {return 0;}
    return 1;   #If date is valid
  }
    
    
    
    
